//+------------------------------------------------------------------+
//|                                           ExtremumPrediction.mq4 |
//|                        Copyright 2012, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//+------------------------------------------------------------------+
#property copyright "Scriptong"
#property link      "http://autograf.dp.ua"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red

extern string A1            = "  ";
extern int    ColBars       = 3; 
extern string A2            = "/  ";
extern bool   SerialDetect  = true;
extern string A3            = " ";
extern bool   ShowCross     = false;
extern string A4            = " ";
extern color  DownLineColor = Red;
extern color  UpLineColor   = Blue;
extern color  CrossColor    = Goldenrod;
extern string A5            = " :  (0 - )   :";
extern int    BeginDay      = 0;
extern int    BeginHour     = 22;
extern int    BeginMin      = 15;
extern string A6            = " :  (0 - )   :";
extern int    EndDay        = 5;
extern int    EndHour       = 21;
extern int    EndMin        = 0;
extern string A7            = "    (0 -  )";
extern int    BarsDepth     = 1000;
extern string A8           = "    ";
extern string prefix        = "ExPr_";

double Sell[];
double Buy[];

int BeginWeek, EndWeek;
datetime LastSignal;

//+-------------------------------------------------------------------------------------+
//| Custom indicator initialization function                                            |
//+-------------------------------------------------------------------------------------+
int init()
{
// - 1 - ==     =========================================
   SetIndexBuffer(0, Buy);
   SetIndexStyle(0, DRAW_ARROW, STYLE_SOLID, 1);   //  " "
   SetIndexArrow(0, 241);
   SetIndexLabel(0, "Buy");
   SetIndexEmptyValue(0, 0.0);   
// - 1 - ==   =============================================================

// - 2 - ==     =========================================
   SetIndexBuffer(1, Sell);
   SetIndexStyle(1, DRAW_ARROW, STYLE_SOLID, 1);   //  " "
   SetIndexArrow(1, 242);
   SetIndexLabel(1, "Sell");
   SetIndexEmptyValue(1, 0.0);
// - 2 - ==   =============================================================

// - 3 - ==     ====================================
   if (ColBars < 2)
   {
      Alert("   ColBars.   2");
      ColBars = 2;
   }
   if (BeginDay < 0 || BeginDay > 6)
   {
      Alert("   BeginDay.   1.");
      BeginDay = 1;
   }
   if (BeginHour < 0 || BeginHour > 23)
   {
      Alert("   BeginHour.   0.");
      BeginHour = 0;
   }
   if (BeginMin < 0 || BeginMin > 59)
   {
      Alert("   BeginMin.   0.");
      BeginMin = 0;
   }
   if (EndDay < 0 || EndDay > 6)
   {
      Alert("   EndDay.   5.");
      EndDay = 5;
   }
   if (EndHour < 0 || EndHour > 23)
   {
      Alert("   EndHour.   22.");
      EndHour = 22;
   }
   if (EndMin < 0 || EndMin > 59)
   {
      Alert("   EndMin.   0.");
      EndMin = 0;
   }
// - 3 - ==   =============================================================
   
// - 4 - ==       -   =============
   BeginWeek = BeginDay*1440 + BeginHour*60 + BeginMin;   
   EndWeek = EndDay*1440 + EndHour*60 + EndMin;  
// - 4 - ==   =============================================================
   
   LastSignal = 0; 
   
   return(0);
}

//+-------------------------------------------------------------------------------------+
//|                                                               |
//+-------------------------------------------------------------------------------------+
void deinit()
{
   for (int i = ObjectsTotal()-1; i >= 0; i--)     //    
      if (StringSubstr(ObjectName(i), 0, StringLen(prefix)) == prefix)
         ObjectDelete(ObjectName(i));
}

//+-------------------------------------------------------------------------------------+
//|                                                       |
//+-------------------------------------------------------------------------------------+
void ShowHLine(datetime time1, datetime time2, double price, color Color)
{
   if (Color == DownLineColor)
      string name = prefix + time2 + "_Down";      //    ..
   else                                            // ..,   ..
      name = prefix + time2 + "_Up";               // .. "_Down"  "_Up"
   if (ObjectFind(name) < 0)                       //      ..
   {                                               // ..,   
      ObjectCreate(name, OBJ_TREND, 0, time1, price, time2, price);
      ObjectSet(name, OBJPROP_RAY, false);
      ObjectSet(name, OBJPROP_COLOR, Color);
      ObjectSet(name, OBJPROP_STYLE, STYLE_SOLID);
   }
}
//+-------------------------------------------------------------------------------------+
//|                                                            |
//+-------------------------------------------------------------------------------------+
void ShowTrendLine(datetime time1, double price1, datetime time2, double price2, 
                   string postfix)
{
   string name = prefix + time2 + postfix ;        //    ..
                                                   // ..,   ..
                                                   // .. postfix
   if (ObjectFind(name) < 0)                       //      ..
   {                                               // ..,   
      ObjectCreate(name, OBJ_TREND, 0, time1, price1, time2, price2);
      ObjectSet(name, OBJPROP_RAY, false);
      ObjectSet(name, OBJPROP_COLOR, CrossColor);
      ObjectSet(name, OBJPROP_STYLE, STYLE_SOLID);
   }
}
//+-------------------------------------------------------------------------------------+
//|                                                             |
//+-------------------------------------------------------------------------------------+
void ShowTime(datetime time, double price, color Color)
{
   string name = prefix + time + price + "_Time";  //    ..
                                                   // ..,  ,
                                                   // ..   "_Time"
   if (ObjectFind(name) < 0)                       //      ..
   {                                               // ..,   
      ObjectCreate(name, OBJ_TEXT, 0, time, price);
      ObjectSet(name, OBJPROP_COLOR, Color);
      ObjectSetText(name, TimeToStr(time, TIME_MINUTES), 8);
   }
}
//+-------------------------------------------------------------------------------------+
//|                                                           |
//+-------------------------------------------------------------------------------------+
void ShowLines(datetime time1, datetime time2, datetime time3, double HighP1, 
               double LowP1, double HighP2, double LowP2, double crossprice, color Color)
{
   ShowHLine(time2, time3, crossprice, Color);
   if (ShowCross)
   {
      ShowTrendLine(time1, LowP1, time2, HighP2, "_1");     
      ShowTrendLine(time1, HighP1, time2, LowP2, "_2");
   }
   ShowTime(time3, crossprice, Color);
}

//+-------------------------------------------------------------------------------------+
//|                                                 |
//+-------------------------------------------------------------------------------------+
double CrossPrice(double y0, double y1, double y2, double y3)
{
   int x2 = ColBars-1;                             //   x ..
                                                   // ..    
                                                   // ...   
                                                   // .. 
   double k1 = (y3 - y0)/x2;                       // .    
   double b1 = y0;                                 // . B   
   double k2 = (y1 - y2)/x2;                       // .    
   double b2 = y2;                                 // . B   
   if (k1 - k2 != 0)                               //    , ..
      double x = (b2 - b1)/(k1 - k2);              // ..   
   else                                            //   
      return(0);   
   return (k1*x + b1);                             //    
}

//+-------------------------------------------------------------------------------------+
//|                  |
//+-------------------------------------------------------------------------------------+
int TrendDuration(double y0, double y1, double y2, double y3, double &y_cross)
{
   y_cross = CrossPrice(y0, y1, y2, y3);           //    
   if (y2 - y1 != 0 && y_cross != 0)               //   ,  ..
      double p = (y_cross - y1)/(y2 - y1);         // ..  ,..
                                                   // .. 
   else                                            //    ..
      return(0);                                   // ..,   0
   if (p != 0)                                     //     ..
      return (MathRound(Period()*(ColBars-1)/p));  // .. ,  ..
                                                   // .. 
   else                                            //     
      return(0);
}
//+-------------------------------------------------------------------------------------+
//|                        |
//+-------------------------------------------------------------------------------------+
datetime TimeOfBar(int cur, int minutes)
{
   datetime dt = Time[cur] + minutes*60;           //    
                                                   // ...  ..
                                                   // ..
   if (Period() < PERIOD_W1)                       //     ..
   {                                               // ..  
      int cur_week_bar = iBarShift(NULL, PERIOD_W1,//     ..
                                   Time[cur]);     // ..,  ..
                                                   // .. 
      datetime week_start = iTime(NULL, PERIOD_W1, //    
                                  cur_week_bar);
      if (dt - week_start > EndWeek*60)            //    ..
                                                   // ..   , ..
         dt += MathCeil((dt - week_start)/1.0/     // .. , ..
                        (PERIOD_W1*60))*           // ..,   
                        (PERIOD_W1 - 
                        (EndWeek - BeginWeek))*60;
   }
   return(dt);
}

//+-------------------------------------------------------------------------------------+
//|                                                             |
//+-------------------------------------------------------------------------------------+
int start()
{
// - 1 - ==    ,      ====
   int cb = IndicatorCounted();                    // - ,   
                                                   // ..   start
   if (cb > 0) cb--;                               //    ..
                                                   // ..
   int limit = Bars - cb;                          //    ,..
                                                   // ..  
   if (BarsDepth != 0 && limit > BarsDepth)        //   - , ..
                                                   // ..  ,..
                                                   // ..    ..
                                                   // .. ,  ..
      limit = BarsDepth;                           // ..   
// - 1 - ==   =============================================================      
   for (int i = limit; i > 0; i--)
   {
// - 2 - ==       ===========================
      if (SerialDetect && Time[i-1] <= LastSignal) //    
         continue;                                 // ..   ..
                                                   // ..   , ..
                                                   // ..  
      int high_bar = iHighest(NULL, 0, MODE_HIGH, ColBars, i);//    
      int low_bar = iLowest(NULL, 0, MODE_LOW, ColBars, i);//    
// - 2 - ==   =============================================================

// - 3 - ==    ==============================================
      if (high_bar == i &&                         //  -   , 
          low_bar == i+ColBars-1 &&                // ..     ..
          Low[i] < High[low_bar])                  // ..    
      {
         double cross_price = 0;                   //    
         double LowP1 = Low[low_bar];              //     ..
         double HighP1 = High[low_bar];            // .. 
         double LowP2 = Low[i];
         double HighP2 = High[i];
         int t = TrendDuration(LowP1, LowP2,       //    , ..
                      HighP1, HighP2, cross_price);//..   
         if (t > 0)                                //   , ..
         {
            LastSignal = TimeOfBar(i, t);          // ..  ..
                                                   // .. , ..
            ShowLines(Time[low_bar], Time[i],      // ..    ..
                      LastSignal, HighP1, LowP1,   // .. 
                      HighP2, LowP2, cross_price, 
                      DownLineColor);
            Sell[i-1] = High[i-1] + iATR(NULL, 0, 14, i-1);//  " "
         }   
      }
// - 3 - ==   =============================================================

// - 4 - ==    ==============================================
      if (high_bar == i+ColBars-1 &&               //  -   ,  -
          low_bar == i &&                          // ..     
          High[i] > Low[high_bar])                 // ..    
      {
         cross_price = 0;                          //    
         LowP1 = Low[high_bar];                    //     ..
         HighP1 = High[high_bar];                  // .. 
         LowP2 = Low[i];
         HighP2 = High[i];
         t = TrendDuration(LowP2, LowP1, HighP2,   //    , ..
                           HighP1, cross_price);   //..   
         if (t > 0)                                //   , ..
         {
            LastSignal = TimeOfBar(i, t);          // ..  ..
                                                   // .. , ..
            ShowLines(Time[high_bar], Time[i],     // ..    ..
                     LastSignal, HighP1, LowP1,    // .. 
                     HighP2, LowP2, cross_price, 
                     UpLineColor);
            Buy[i-1] = Low[i-1] - iATR(NULL, 0, 14, i-1);//  " "
         }   
      }
// - 4 - ==   =============================================================
   }
   WindowRedraw();
   return(0);
}